@@ -1,11 +1,14 @@
 "Simple parser for Garmin TCX files."
 import time
+from datetime import timedelta
 
+from dateutil.parser import isoparse
 from lxml import objectify
 
 from .exceptions import NoHeartRateDataError
 
 namespace = "http://www.garmin.com/xmlschemas/TrainingCenterDatabase/v2"
+namespace2 = "http://www.garmin.com/xmlschemas/ActivityExtension/v2"
 
 
 class TCXParser:
@@ -48,12 +51,44 @@ class TCXParser:
             x.text for x in self.root.xpath("//ns:Time", namespaces={"ns": namespace})
         ]
 
+    def time_objects(self):
+        """Return a list of timezone-aware datetime objects for each string in time_values()."""
+        return [isoparse(x) for x in self.time_values()]
+
+    def time_durations(self):
+        """Return a list of timedelta objects for each string in time_values().
+
+        This can be used to calculate the total time in each HR zone, as opposed to the
+        total amount of trackpoints in each HR zone.
+        """
+        time_objects = self.time_objects()
+        last_index = len(time_objects) - 1
+
+        durations = []
+        for idx, dt in enumerate(time_objects):
+            if idx > 0:
+                # Add half the time elapsed since previous trackpoint
+                duration = (dt - time_objects[idx - 1]) / 2
+            else:
+                duration = timedelta(0)
+            if idx < last_index:
+                # Add half the time to next trackpoint
+                duration += (time_objects[idx + 1] - dt) / 2
+            durations.append(duration)
+        return durations
+
     def cadence_values(self):
         return [
             int(x.text)
             for x in self.root.xpath("//ns:Cadence", namespaces={"ns": namespace})
         ]
 
+    def power_values(self):
+        return [
+            int(x.text)
+            for x in self.root.xpath("//ns:TPX/ns:Watts", namespaces={"ns": namespace2})
+        ]
+
     @property
     def latitude(self):
         if hasattr(self.activity.Lap.Track.Trackpoint, "Position"):
@@ -78,7 +113,11 @@ class TCXParser:
 
     @property
     def cadence_avg(self):
-        return self.activity.Lap[-1].Cadence
+        return (
+            self.activity.Lap[-1].Cadence
+            if hasattr(self.activity.Lap[-1], "Cadence")
+            else None
+        )
 
     @property
     def distance(self):
@@ -102,17 +141,17 @@ class TCXParser:
     def hr_avg(self):
         """Average heart rate of the workout"""
         hr_data = self.hr_values()
-        return int(sum(hr_data) / len(hr_data))
+        return int(sum(hr_data) / len(hr_data)) if hr_data else None
 
     @property
     def hr_max(self):
         """Maximum heart rate of the workout"""
-        return max(self.hr_values())
+        return max(self.hr_values()) if self.hr_values() else None
 
     @property
     def hr_min(self):
         """Minimum heart rate of the workout"""
-        return min(self.hr_values())
+        return min(self.hr_values()) if self.hr_values() else None
 
     def hr_percent_in_zones(self, zones):
         """Percentage of workout spent in each heart rate zone.
@@ -152,6 +191,38 @@ class TCXParser:
             per_zone[name] = round(100 * count / nr_hr_values)
         return per_zone
 
+    def hr_time_in_zones(self, zones):
+        """Time spent in each heart rate zone.
+
+        Given these user's heart rate zones:
+        zones = {
+            "Z0": (0, 119),
+            "Z1": (120, 199),
+            "Z2": (200, 240),
+        }
+
+        Then `self.hr_time_in_zones(zones)` would return something like:
+        {
+            "Z0": timedelta(seconds=3600),
+            "Z1": timedelta(seconds=1800),
+            "Z2": timedelta(seconds=300),
+        }
+        """
+        hr_values = self.hr_values()
+        if not hr_values:
+            raise NoHeartRateDataError
+
+        # Initialize a dictionary with time=0 for each zone
+        per_zone = dict.fromkeys(zones.keys(), timedelta(0))
+
+        # count number of HR measurements per zone
+        for hr, td in zip(hr_values, self.time_durations()):
+            for zone_name, zone_boundaries in zones.items():
+                if hr >= zone_boundaries[0] and hr <= zone_boundaries[1]:
+                    per_zone[zone_name] += td
+
+        return per_zone
+
     @property
     def pace(self):
         """Average pace (mm:ss/km for the workout"""
@@ -202,9 +273,21 @@ class TCXParser:
     def cadence_max(self):
         """Returns max cadence of workout"""
         cadence_data = self.cadence_values()
-        return max(cadence_data)
+        return max(cadence_data) if cadence_data else None
 
     @property
     def activity_notes(self):
         """Return contents of Activity/Notes field if it exists."""
         return getattr(self.activity, "Notes", "")
+
+    @property
+    def power_max(self):
+        """Returns max power (in watts) of workout"""
+        power_data = self.power_values()
+        return max(power_data) if power_data else None
+
+    @property
+    def power_avg(self):
+        """Returns avg power (in watts) of workout"""
+        power_data = self.power_values()
+        return int((sum(power_data) / len(power_data))) if power_data else None
